﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Collections.Specialized;
using BayesNetwork.Classes;

namespace Project.Classes
{
    class Agent : EnvironmentObject
    {
        public double hungry = 30;
        public const int MaxStorage = 50;
        private Dictionary<string, bool> hunts = new Dictionary<string,bool>();
        private Dictionary<string, bool> gathers = new Dictionary<string, bool>();
        public Dictionary<string, int> Inventory = new Dictionary<string, int>();
        public int AmountInventory = 0;
        public EnvironmentObject Home;
        public string goal;
        private Stack<int[]> path = null;
        private Astar astar = new Astar();
        // social related variables
        public List<Agent> Family = new List<Agent>();

        BayesNet bnet;

        public Agent(BayesNet Bnet, EnvironmentObject home)
        {
            this.bnet = Bnet;
            this.name = "Agent";
            this.category = "Animal";
            this.Speed = 1;
            this.iCanMove = true;
            this.cannotBeWith.Add("water", true);
            this.chanceToDuplicate = 0.8;
            this.priority = 15;
            this.resouces.Add("meat", 25);
            this.maxSquare = 500;
            this.goal = "hunt";
            if (home.name.Equals("Cave"))
            {
                Cave cave = (Cave)home;
                if (cave.moveIn(this))
                    this.Home = cave;
                else
                    this.goal = "findNewHouse";
            }
            else if (home.name.Equals("House"))
            {
                House house = (House)home;
                if (house.moveIn(this))
                    this.Home = house;
                else
                    this.goal = "findNewHouse";
            }
            hunts.Add("rabbit", true);
            gathers.Add("tree", true);
            gathers.Add("mountain", true);

            this.Family.Add(this);
        }

        public override bool isalive()
        {
            if ( hungry < 0 || age > 60)
            {
                this.iCanMove = false;
                this.Die();
                EnvironmentMap.remove(this, X, Y);
                return true;
            }
            else
            {
                return false;
            }
        }

        public override string toString()
        {
            return "A";
        }

        private bool amIAtHome()
        {
            HashSet<EnvironmentObject> O = EnvironmentMap.getAll(X, Y);
            foreach (EnvironmentObject E in O)
                if (E == Home)
                    return true;
            return false;
        }

        public bool collectWood()
        {
            if (AmountInventory < MaxStorage)
            {
                HashSet<EnvironmentObject> O = EnvironmentMap.getAll(X, Y);
                foreach (EnvironmentObject E in O)
                {
                    int RemaningInvetory = MaxStorage - AmountInventory;
                    if (RemaningInvetory <= 0) // hopefully it will never be less then 0
                        return true;
                    if (gathers.ContainsKey(E.name))
                        foreach (KeyValuePair<string, int> resource in E.resouces)
                            if (resource.Key.Equals("wood"))
                            {
                                //Console.WriteLine("I hunted a(n) " + E.name + " at " + X + " & " + Y);
                                // I hunted you
                                int Gathered = Math.Min(RemaningInvetory, resource.Value);
                                if (!Inventory.ContainsKey("wood"))
                                    Inventory.Add("wood", Gathered);
                                else
                                    Inventory["wood"] += Gathered;
                                AmountInventory += Gathered;

                                E.iCanMove = false;
                                EnvironmentMap.remove(E, E.X, E.Y);
                            }
                }
            }
            return true;
        }

        public bool collectStone()
        {
            if (AmountInventory < MaxStorage)
            {
                HashSet<EnvironmentObject> O = EnvironmentMap.getAll(X, Y);
                foreach (EnvironmentObject E in O)
                {
                    int RemaningInvetory = MaxStorage - AmountInventory;
                    if (RemaningInvetory <= 0) // hopefully it will never be less then 0
                        return true;
                    if (gathers.ContainsKey(E.name))
                        foreach (KeyValuePair<string, int> resource in E.resouces)
                            if (resource.Key.Equals("stone"))
                            {
                                //Console.WriteLine("I hunted a(n) " + E.name + " at " + X + " & " + Y);
                                // I hunted you
                                int Gathered = Math.Min(RemaningInvetory, resource.Value);
                                if (!Inventory.ContainsKey("stone"))
                                    Inventory.Add("stone", Gathered);
                                else
                                    Inventory["stone"] += Gathered;
                                AmountInventory += Gathered;

                                E.resouces["stone"] -= Gathered;
                                return true;
                            }
                }
            }
            return true;
        }

        public bool huntFood()
        {
            if (AmountInventory < MaxStorage)
            {
                HashSet<EnvironmentObject> O = EnvironmentMap.getAll(X, Y);
                foreach (EnvironmentObject E in O)
                {
                    int RemaningInventory = MaxStorage - AmountInventory;
                    if (RemaningInventory <= 0) // hopefully it will never be less then 0
                        return true;
                    if (hunts.ContainsKey(E.name))
                        foreach (KeyValuePair<string, int> resource in E.resouces)
                            if (resource.Key.Equals("meat"))
                            {
                                //Console.WriteLine("I hunted a(n) " + E.name + " at " + X + " & " + Y);
                                // I hunted you
                                int Gathered = Math.Min(RemaningInventory, resource.Value);
                                if (!Inventory.ContainsKey("meat"))
                                    Inventory.Add("meat", Gathered);
                                else
                                    Inventory["meat"] += Gathered;
                                AmountInventory += Gathered;

                                E.iCanMove = false;
                                EnvironmentMap.remove(E, E.X, E.Y);
                            }
                }
            }
            
            return true;
        }


        public override bool update()
        {
            this.think();
            return base.update();
        }

        private void think()
        {
            //Console.WriteLine("My current goal is: " + this.goal);
            --hungry;

            if (this.goal.Equals("findNewHouse"))
            {
                HashSet<EnvironmentObject> O = EnvironmentMap.getAll(X, Y);
                // is there a house here?
                foreach (EnvironmentObject E in O)
                    if (E.name.Equals("Cave"))
                    {
                        // we found a cave... try to move in
                        Cave cave = (Cave)E;
                        if (cave.moveIn(this))
                        {
                            //Console.WriteLine("Found a new home!");
                            this.Home = cave;
                            this.goal = "hunt";
                        }
                    }
                    else if (E.name.Equals("House"))
                    {
                        // we found a house... try to move in
                        House house = (House)E;
                        if (house.moveIn(this))
                        {
                            //Console.WriteLine("Found a new home!");
                            this.Home = house;
                            this.goal = "hunt";
                        }
                    }
                
                return;
            }

            if (this.goal.Equals("buildHouse"))
            {
                this.buildHouse();
                return;
            }

            if (this.goal.Equals("collectWood"))
                this.collectWood();

            if (this.goal.Equals("collectStone"))
                this.collectStone();

            if (this.goal.Equals("hunt"))
                this.huntFood();
            
            if(this.goal.Equals("stayHome"))
            {
                if (Home.name.Equals("Cave"))
                {
                    Cave home = (Cave)Home;
                    home.relax(this);
                }
                else if (Home.name.Equals("House"))
                {
                    House house = (House)Home;
                    house.relax(this);
                }
            }

            if (!this.goal.Equals("goHome") && Home != null)
            {
                if ((MaxStorage - AmountInventory) < 10)
                {
                    //Console.WriteLine("My inventory is at " + AmountInventory + ". I am going home.");
                    path = astar.findPath(this, Home.X, Home.Y);
                    this.goal = "goHome"; // You are running out of space... go back home
                }
                else if (hungry <= 15)
                {
                    //Console.WriteLine("My hunger is at " + hungry + ". I am going home.");
                    path = astar.findPath(this, Home.X, Home.Y);
                    this.goal = "goHome"; // you are hungry and need to be home to eat
                }
            }
            else
                this.huntFood(); // while heading home... if we see somthing to kill lets kill it
        }

        public override bool move()
        {
            if (this.goal.Equals("goHome") && Home != null)
            {
                if (path.Count <= 0)
                {
                    if (!amIAtHome())
                        path = astar.findPath(this, Home.X, Home.Y);
                    else
                        this.goal = "stayHome";
                    return true;
                } 
                int[] next = path.Pop();

                EnvironmentMap.move(this, next[0], next[1]);
                return true;
            }
            else
                return base.move();
        }

        private bool buildHouse()
        {
            bool goodPlace = false;
            HashSet<EnvironmentObject> O = EnvironmentMap.getAll(X, Y);
            foreach (EnvironmentObject E in O)
                if (E.name.Equals("tree"))
                    goodPlace = true;
            if (this.hungry <= 10)
                // i am to hungry to care
                goodPlace = true;
            if(goodPlace)
            {
                House house = new House(this.bnet);
                EnvironmentMap.add(house, X, Y);
                house.moveIn(this);
                this.Home = house;
                this.goal = "hunt";
            }
            else
                return false;
            return true;
        }

        public bool addFamily(Agent a)
        {
            if (Family.Contains(a) || a.Family.Contains(this))
                return false;

            Family.Add(a);
            a.Family.Add(this);
            return true;
        }

        public bool addFamily(List<Agent> List)
        {
            List<Agent> list = new List<Agent>(List);
            foreach (Agent a in list)
            {
                if (!this.Family.Contains(a))
                {
                    this.Family.Add(a);
                    a.Family.Add(a);
                }
            }
            return true;
        }

        public override bool duplicate()
        {
            if (age < 18 || age > 50 || 
                hungry < 5)
                return false;

            HashSet<EnvironmentObject> O = EnvironmentMap.getAll(X, Y);
            double chance;
            bool other = false;

            foreach (EnvironmentObject E in O)
                // if there are other agents around.... try to mate with them
                if (E.name.Equals(this.name) && E.age > 20)
                    other = true;
                
            if (other && amIAtHome() && EnoughFood())
            {
                chance = EnvironmentMap.Rand.NextDouble();
                if (chance < chanceToDuplicate)
                {
                    Agent a = new Agent(bnet, this.Home);
                    a.addFamily(Family);
                    EnvironmentMap.add(a, X, Y);
                }
            }

            return true;
        }

        private bool EnoughFood()
        {
            if (this.Home.name.Equals("Cave"))
            {
                Cave cave = (Cave)this.Home;
                if (cave.Inventory.ContainsKey("meat") 
                    && cave.Inventory["meat"] > 50)
                    return true;
            }
            else if (this.Home.name.Equals("House"))
            {
                House house = (House)this.Home;
                if (house.Inventory.ContainsKey("meat")
                    && house.Inventory["meat"] > 50)
                    return true;
            }
            return false;
        }

        public override void Die()
        {
            // when an agent dies it "moves out" of its home
            if (Home != null && Home.name.Equals("Cave"))
            {
                Cave cave = (Cave)Home;
                cave.moveOut(this);
            }
            // well i'm dead... my family does not care for me anymore :P
            foreach(Agent a in Family)
                if(a != this)
                    a.Family.Remove(this);
            Family.Clear();

            base.Die();
        }
    }
}
